@@ -18,9 +18,12 @@ int semctl(int id, int num, int cmd, ...)
 {
 	union semun arg = {0};
 	va_list ap;
-	switch (cmd) {
-	case SETVAL: case GETALL: case SETALL: case IPC_STAT: case IPC_SET:
-	case IPC_INFO: case SEM_INFO: case SEM_STAT:
+	switch (cmd & ~IPC_TIME64) {
+	case SETVAL: case GETALL: case SETALL: case IPC_SET:
+	case IPC_INFO: case SEM_INFO:
+	case IPC_STAT & ~IPC_TIME64:
+	case SEM_STAT & ~IPC_TIME64:
+	case SEM_STAT_ANY & ~IPC_TIME64:
 		va_start(ap, cmd);
 		arg = va_arg(ap, union semun);
 		va_end(ap);
@@ -34,17 +37,23 @@ int semctl(int id, int num, int cmd, ...)
 	}
 #endif
 #ifndef SYS_ipc
-	int r = __syscall(SYS_semctl, id, num, cmd | IPC_64, arg.buf);
+	int r = __syscall(SYS_semctl, id, num, IPC_CMD(cmd), arg.buf);
 #else
-	int r = __syscall(SYS_ipc, IPCOP_semctl, id, num, cmd | IPC_64, &arg.buf);
+	int r = __syscall(SYS_ipc, IPCOP_semctl, id, num, IPC_CMD(cmd), &arg.buf);
 #endif
 #ifdef SYSCALL_IPC_BROKEN_MODE
-	if (r >= 0) switch (cmd) {
+	if (r >= 0) switch (cmd | IPC_TIME64) {
 	case IPC_STAT:
 	case SEM_STAT:
 	case SEM_STAT_ANY:
 		arg.buf->sem_perm.mode >>= 16;
 	}
+#endif
+#if IPC_TIME64
+	if (r >= 0 && (cmd&IPC_TIME64)) {
+		IPC_HILO(arg.buf, sem_otime);
+		IPC_HILO(arg.buf, sem_ctime);
+	}
 #endif
 	return __syscall_ret(r);
 }
